  /********************************************************/
  /*                                                      */
  /*   HOSTMODM.C - Modem-linked Host Communication       */
  /*                                                      */
  /*            Copyright (c) 1998                        */
  /*            Triangle Research Int'l Pte Ltd           */
  /*                                                      */
  /********************************************************/



#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <time.h>
#include <string.h>
#include <dos.h>
#define lo(f) ((f) & 0xff)
#define hi(f) (lo(f>>8))

int lookkey(int *k);
char getkey(int *k);
char getkeyc(void);
void beep(void);

#define SPBAR           0x3920
#define BKSP      	0x0e08
#define R_ARROW   	0x4d00
#define L_ARROW   	0x4b00
#define UP_ARROW  	0x4800
#define DN_ARROW  	0x5000
#define CTRL_R_ARROW    0x7400
#define CTRL_L_ARROW    0x7300
#define CTRL_HOME       0x7700
#define CTRL_END        0x7500


#define PG_UP     0x4900
#define PG_DN     0x5100
#define HOME      0x4700
#define END       0x4f00
#define ESCkey    0x011b
#define RET       0x1c0d
#define TAB       0x0f09

#define INS       0x5200
#define DEL       0x5300

#define F1        0x3b00
#define F2        0x3c00
#define F3        0x3d00
#define F4        0x3e00
#define F5        0x3f00
#define F6        0x4000
#define F7        0x4100
#define F8        0x4200
#define F9        0x4300
#define F10       0x4400

#define ALT_C     0x2E00
#define ALT_E     0x1200
#define ALT_F     0x2100
#define ALT_H     0x2300
#define ALT_I     0x1700
#define ALT_O     0x1800
#define ALT_P     0x1900
#define ALT_R     0x1300
#define ALT_S     0x1f00
#define ALT_T     0x1400
#define ALT_X     0x2D00

#define ZERO      0x0B30
#define ONE       0x0231
#define TWO       0x0332
#define THREE     0x0433
#define FOUR      0x0534
#define FIVE      0x0635
#define SIX       0x0736
#define SEVEN     0x0837
#define EIGHT     0x0938
#define NINE      0x0A39

#define NUMZERO      0x5230
#define NUMONE       0x4F31
#define NUMTWO       0x5032
#define NUMTHREE     0x5133
#define NUMFOUR      0x4B34
#define NUMFIVE      0x4C35
#define NUMSIX       0x4D36
#define NUMSEVEN     0x4737
#define NUMEIGHT     0x4838
#define NUMNINE      0x4939

#define CTRL_PGUP    0x8400
#define CTRL_PGDN    0x7600
#define CTRL_RET     0x1C0A
#define CTRL_F7      0x6400
#define CTRL_F8      0x6500
#define CTRL_F9      0x6600
#define CTRL_BKSP    0x0e7f

#define CTRL_T       0x1414
#define CTRL_M       0x320D
#define CTRL_L       0x260C
#define CTRL_R       0x1312
#define CTRL_S       0x1F13
#define CTRL_C       0x2E03
#define CTRL_I       0x1709
#define CTRL_P       0x1910
#define CTRL_N       0x310E
#define CTRL_B       0x3002
#define CTRL_H       0x2308
#define CTRL_F       0x2106
#define NUMPLUS      0x4e2b
#define NUMMINUS     0x4a2d



int lookkey(int *k) {
  union REGS regs;
  regs.h.ah = 1;

  int86(0x16,&regs,&regs);
  *k = regs.x.ax;
  if (regs.x.flags & 0x0040) return(0);
  else return(1);
}

char getkey(int *k) {
  union REGS regs;
  int m;

  while (1) {
    if (lookkey(k)!=0) {
       regs.h.ah = 0;
       int86(0x16,&regs,&regs);
       *k = regs.x.ax;
       return regs.h.al;
    }
  }
}

void transmit485(void);
void receive485(void);
int serial_getc(void);
void serial_out(unsigned char ch);
void init_serial(void);
int serial_in();
int serial_status();
void comm_error(void);

int commPort,commStatus,commData, commControl, commLineControl;
char command[20], response[100];

/*-------------------------------------------------------------------------*/


void main()
{
  char temp[10];
  char ch;
  int i,k;

  printf("\nModem at COM port (1 or 2)?>");
  scanf("%d",&ch);
  if (ch==1)  commPort = 0;    /* commPort = 0 for COM1: */
  else commPort = 1;           /* commPort = 1 for COM2: */

  ch=getchar(); 	       /* clear keyboard buffer */
  init_serial();               /* initialize PC's serial port */
  clrscr();                    /* clear PC's screen */

  printf("   Use a Modem to Send Multi-point command to TRi's PLC\n");
  printf("   Baud rate = 9,600 bps \n");
  printf("   ==============================================================\n");
  printf("   <F1>-Dial    <F9>-Hang Up   <ESC>-Quit Program\n\n");


  while (1) {
    printf("Command String ? ");
    while(lookkey(&k)==0);
    switch(k) {
        case F1    :  getkey(&k);
                      dialup();
                      continue;
        case F9    :  getkey(&k);
                      hangup();
                      continue;
        case ESCkey:  getkey(&k);
                      quit();
                      continue;
    }
    gets(command);
    while(serial_in());     /* clear all characters from buffer */
    if (send_command(command, response)==-1) comm_error();
    else printf("Response String: %s\n\n",response);
  }
}

send_modem(char *string)     /* send command to modem */
{
  int i;
  int ch;

  if (commStatus==0) init_serial();
  for (i=0;string[i]!=0;i++) serial_out(string[i]);
  serial_out('\r');
  serial_out('\n');
}

get_modem(char * string)             /* obtain a string from modem */
{
    int i, ch;
    string[0]=0;                     /* clear the string */
    for (i=0;i<100;i++)              /* wait to receive response */
    {
       if ((ch=serial_getc()) == -1)
	  return (-1);
       string[i]=ch;
       if (ch=='\r') {
	  ch=serial_getc();
	  string[i]=0;
	  break;
       }
    }
    return (1);
}

/*-------------------------------------------------------------------------*/
/* You must configure the modem for no-local echo and ignore DTR           */
/* Example here tested on a Compaq 336 modem and Motorola Lifestyle        */
/* modem.                                                                  */
/* One of the two modem must be set to connect at 9600 only                */
/*
/*-------------------------------------------------------------------------*/

dialup()
{
  int k;
  char temp[10];

  send_modem("ATE0");           /* No echo from modem */
  get_modem(response);
  send_modem("ATQ\Q%C&D");      /* disable flow control, ignore DTR */
                                /* disable compression */
  get_modem(response);
  printf("\rEnter Phone Number to dial: ");
  gets(temp);
  while (serial_in());
  strcpy(command, "ATDT");
  strcat(command, temp);

  printf("Dialling: ATDT%s\n",temp);
  send_modem(command);
  while (1) {
    if (lookkey(&k)){                  /* ESC key stop it */
      getkey(&k);
      if (k==ESCkey) break;
    }
    if (get_modem(response)==1) {
      if (strncmp(response,"CONNECT",7)==0) {  /* connected */
         end:
         printf("%s",response);
         break;
      }
      if (strncmp(response,"NO",2)==0) goto end; /* No carrier */
      if (strncmp(response,"BUSY",4)==0) goto end; /* Line busy */

    }
  }
  printf("\n");
}

hangup()
{
  int ch;

  printf("\rDisconnect and Hang Up the Phone (y/n)?");
  if ((ch=getche())=='y' || ch=='Y') {
    printf("\n+++");
    serial_strout("+++");           /* escape to command mode */
    delay(1200);
    serial_out('\r');
    serial_out('\n');
    printf("\nATH\n");
    get_modem(response);

    send_modem("ATH");  /* command to hang up the phone */
    delay(1000);
    get_modem(response);
    printf("Modem reply:%s\n",response);

  }
  else printf("\n");
}

quit()
{
  int ch;
  printf("\rQuit the program (y/n)?");
  if ((ch=getche())=='y' || ch=='Y') exit(0);
  else printf("\n");
}

/*-------------------------------------------------------------------------*/
/*  This function establish a Point-to-point or Multipoint communication   */
/*  with the target controller.                                            */
/*                                                                         */
/*  For point to point communication, it sends a Ctrl-E character and      */
/*  waits to obtain echo of Ctrl-E from the controller. For a multipoint   */
/*  command (string starts with '@' character) it will not send Ctrl-E     */
/*                                                                         */
/*  The command "string" is sent to the controller via the                 */
/*  serial port. It will then wait to receive the response from the        */
/*  controller and store it in the "response" string.                      */
/*                                                                         */
/*  Function return (-1) if communication error occurs                     */
/*-------------------------------------------------------------------------*/


int send_command(char *string,char *respstr)
{
    int i;
    int ch;

    if (commStatus==0) init_serial();
    for (i=0;string[i]!=0;i++) serial_out(string[i]);
    serial_out('\r');
    respstr[0]=0;                     /* clear the string */
    for (i=0;i<100;i++)              /* wait to receive response */
    {
       if ((ch=serial_getc()) == -1)
	  return (-1);
       respstr[i]=ch;
       if (ch=='\r') {
	  respstr[i]=0;
	  break;
       }
    }
    return (1);
}

/*-------------------------------------------------------------------------*/
/* This function check the serial port "commPort" (COM1 = 0; COM2 = 1) for */
/* data. It returns the 7-bit data if ready.                               */
/* If after many tries still no data ready, function return (-1) to        */
/* indicate time-out error. This is to prevent locking up the computer in  */
/* a dead loop if device does not respond.                                 */
/* You should change the retry time to match the speed of your computer    */
/*-------------------------------------------------------------------------*/

int serial_getc(void)
{
   int i,j;
   for (j=0;j<10;j++) {
     for (i=0;i<10000;i++) {                 /* allow for time out */
	if (serial_status())
	    return (serial_in());
     }
   }
   return(-1);                           /* nothing received, time-out !! */
}
/*-------------------------------------------------------------------------*/
/* Output a character to the serial port                                   */
/*-------------------------------------------------------------------------*/

void serial_out(unsigned char ch) {
  outportb(commData,ch);
  while (!(inportb(commStatus) & 0x20))
       ;
}

serial_strout(unsigned char *str)
{
  while (*str) {
     serial_out(*str++);
  }
}

/*-------------------------------------------------------------------------*/
/*  Initialize the IBMPC serial communication port to 38400, 8 data bits,   */
/*  1 stop bit and no parity.                                              */
/*  must be called once before communicating with T44P                     */
/*-------------------------------------------------------------------------*/

#define SETTINGS (0x03|0x00|0x00|0xE0)

void init_serial(void)
{
   bioscom(0,SETTINGS,commPort);   /* for non 100% IBM compatible use */
   set_baud_rate(3);         /* initialize to 9600 baud, 8 data, 1 stop, no parity */
   outportb(commControl,3);  /* Turn on /RTS and /DTR */
}

/*--------------------------------------------------------------------------*/
/* The baud rate of the PC serial port can be changed by the following      */
/* function using a divisor to divide the baud rate. The divisor value for  */
/* each supported baud rate is contained in the array "divisor[]".          */
/*    The input argument "baud_select" determines the baud rate as follow:  */
/*         Baud rate        baud_select                                     */
/*         =========        ===========                                     */
/*          1200               0                                            */
/*          2400               1                                            */
/*          4800               2                                            */
/*          9600               3                                            */
/*          19200              4                                            */
/*          38400              5                                            */
/*--------------------------------------------------------------------------*/

int divisor[]={ 96,48,24,12,6,3 };

set_baud_rate(int baud_select)
{
   if (commPort==0)
   {
      commStatus = 0x3FD;
      commData   = 0x3F8;
      commControl= 0x3FC;
      commLineControl = 0x3FB;
   }
   else
   {
      commStatus = 0x2FD;
      commData   = 0x2F8;
      commControl= 0x2FC;
      commLineControl = 0x2FB;
   }
   outportb(commLineControl,inportb(commLineControl) | 0x80);  /* set DLAB */
   outportb(commData, divisor[baud_select]);
   outportb(commData+1, 0);
   outportb(commLineControl,inportb(commLineControl) & 0x7F);  /* clear DLAB*/

   outportb(commControl,1);    /* Force /DTR to low for Opto232 use */
   while (serial_in());        /* Flush all characters in serial buffer */
}
/*-------------------------------------------------------------------------*/
/*  If a character is ready at the 8250 UART, return the character.        */
/*  Otherwise return 0.                                                    */
/*-------------------------------------------------------------------------*/

int serial_in() {
  if (inportb(commStatus) & 0x01) return(inportb(commData));
  return(0);
}
/*-------------------------------------------------------------------------*/
/*  Check 8250 UART status.                                                */
/*    returns 1 if a character is present, otherwise returns 0             */
/*-------------------------------------------------------------------------*/

int serial_status() {
  char ch;
  ch = inportb(commStatus);
  if ((ch & 0x0F) != 0)
     return (-1);
  return (inportb(commStatus) & 0x01);
}
/*-------------------------------------------------------------------------*/
/*  Report to user that a communication error occur between the computer   */
/*  and the controller                                                     */
/*-------------------------------------------------------------------------*/

void comm_error(void)
{
  int k;
  printf(
    "\n\tCommunication Error!... Check serial port COM%d:\n\n",commPort+1);
}
/*-------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------*/
